
#!/usr/bin/env python
###############################################################################
#                                                                             #
# This program is free software: you can redistribute it and/or modify        #
# it under the terms of the GNU General Public License as published by        #
# the Free Software Foundation, either version 3 of the License, or           #
# (at your option) any later version.                                         #
#                                                                             #
# This program is distributed in the hope that it will be useful,             #
# but WITHOUT ANY WARRANTY; without even the implied warranty of              #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the                #
# GNU General Public License for more details.                                #
#                                                                             #
# You should have received a copy of the GNU General Public License           #
# along with this program. If not, see <http://www.gnu.org/licenses/>.        #
#                                                                             #
###############################################################################
 
__author__ = "Joel Boyd"
__copyright__ = "Copyright 2017"
__credits__ = ["Joel Boyd"]
__license__ = "GPL3"
__version__ = "0.0.1"
__maintainer__ = "Joel Boyd"
__email__ = "joel.boyd near uq.net.au"
__status__ = "Development"
 
###############################################################################
# System imports

class GffGenerator():

	def write(self, genome, output_file):
		'''
		Writes a gff file for a Genome object
		
		Parameters
		----------
		genome 			- Genome object
		output_file 	- string. file name to output results to.
		'''

		with open(output_file, 'w') as out_io:
			for sequence in genome.ordered_sequences():
				contig_id = '_'.join(sequence.seqname.split('_')[:-1])
				
				features  = ['seq_id=%s'		% sequence.seqname,
						 	 'prodigal_id=%s' 	% sequence.prod_id,
							 'partial=%s' 	 	% sequence.partial,
							 'start_type=%s'  	% sequence.starttype,
							 'rbs_motif=%s'   	% sequence.rbs_motif,
							 'rbs_spacer=%s'  	% sequence.rbs_spacer,
							 'gc=%s'    		% sequence.gc]

				if len(sequence.all_annotations())>0:
					features.append('annotations=%s' % ','.join(sequence.all_annotations()))
				else:
					features.append('annotations=hypothetical_protein')

				line = [contig_id, 
						'prodigal',
						'CDS',
						sequence.startpos,
						sequence.finishpos,
						'.',
						('-' if sequence.direction == '-1'
						 else '+'),
						'0',
						';'.join(features)]
						
				out_io.write('\t'.join(line) + '\n')


#!/usr/bin/env python
###############################################################################
#                                                                             #
# This program is free software: you can redistribute it and/or modify        #
# it under the terms of the GNU General Public License as published by        #
# the Free Software Foundation, either version 3 of the License, or           #
# (at your option) any later version.                                         #
#                                                                             #
# This program is distributed in the hope that it will be useful,             #
# but WITHOUT ANY WARRANTY; without even the implied warranty of              #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the                #
# GNU General Public License for more details.                                #
#                                                                             #
# You should have received a copy of the GNU General Public License           #
# along with this program. If not, see <http://www.gnu.org/licenses/>.        #
#                                                                             #
###############################################################################
 
__author__ = "Joel Boyd"
__copyright__ = "Copyright 2017"
__credits__ = ["Joel Boyd"]
__license__ = "GPL3"
__version__ = "0.0.1"
__maintainer__ = "Joel Boyd"
__email__ = "joel.boyd near uq.net.au"
__status__ = "Development"

###############################################################################

import logging
import os

from kegg_matrix import KeggMatrix
from network_builder import NetworkBuilder

###############################################################################

class NetworkAnalyser:
    
    MATRIX          = 'matrix'
    NETWORK         = 'network'
    EXPLORE         = 'explore'
    DEGRADE         = 'degrade'
    PATHWAY         = 'pathway'
    ANNOTATE        = 'annotate'
    ENRICHMENT      = 'enrichment'
    MODULE_AB       = 'module_ab'
    TRAVERSE        = 'traverse'


    NETWORK_OUTPUT_FILE  = 'network.tsv'
    METADATA_OUTPUT_FILE = 'metadata.tsv'    
    TRAVERSE_OUTPUT_FILE = 'traverse.tsv'    

    def __init__(self, metadata):
        self.metadata = {}
        for line in open(metadata):
            if line.startswith('#'):continue
            sample_id, group = line.strip().split('\t')
            if group in self.metadata:
                self.metadata[group].append(sample_id)
            else:
                self.metadata[group] = [sample_id]

    def _write_results(self, output_path, output_lines):
        '''
        Parameters
        ----------
        output_path: string
            Path to non-existent file to write output lines to
        output_lines: list
            list containing lines to write to output path
        '''
        logging.info('Writing results to file: %s' % output_path)
        with open(output_path, 'w') as output_path_io: 
            output_path_io.write('\n'.join(output_lines))
            output_path_io.flush()
        
    def do(self, matrix, transcriptome, metabolome, depth, filter, limit, queries, 
           subparser_name, starting_compounds, steps, number_of_queries, output_directory):
        '''
        Parameters
        ----------
        depth
        filter
        limit
        metabolome
        queries

        subparser_name
        transcriptome
        output_directory

        '''

        nb = NetworkBuilder(self.metadata.keys())
        km = KeggMatrix(matrix, transcriptome)

        abundances_metagenome = \
                {key:km.group_abundances(self.metadata[key],
                                         km.reaction_matrix) 
                 for key in self.metadata.keys()}

        if transcriptome:
            abundances_transcriptome = \
                    {key:km.group_abundances(self.metadata[key],
                                             km.reaction_matrix_transcriptome) 
                     for key in self.metadata.keys()}            
            abundances_expression = \
                    {key:km.group_abundances(self.metadata[key],
                                             km.reaction_matrix_expression) 
                     for key in self.metadata.keys()}
        else:
            abundances_transcriptome = None
            abundances_expression    = None

        if metabolome:
            abundances_metabolome = Matrix(metabolome) 
            ### ~ TODO: This WILL NOT WORK - MATRIX is no longer an existing class.
            ### ~ TODO: I've added a note in the holp for network analyzer 
            ### ~ TODO: that warns the user about this.
        else:
            abundances_metabolome = None

        if subparser_name==self.TRAVERSE:
            logging.info('Traversing network')
            output_lines = \
                            nb.traverse(abundances_metagenome,
                                        abundances_transcriptome,
                                        limit,
                                        filter,
                                        starting_compounds,
                                        steps,
                                        number_of_queries)
            self._write_results(os.path.join(output_directory, self.TRAVERSE_OUTPUT_FILE), output_lines)

        elif subparser_name==self.EXPLORE:
            logging.info("Using supplied queries (%s) to explore network" \
                                                        % queries)
            network_lines, node_metadata = \
                            nb.query_matrix(abundances_metagenome, 
                                            abundances_transcriptome,
                                            abundances_expression,
                                            queries,
                                            depth)

            self._write_results(os.path.join(output_directory, self.NETWORK_OUTPUT_FILE), network_lines)
            self._write_results(os.path.join(output_directory, self.METADATA_OUTPUT_FILE), node_metadata)

        elif subparser_name==self.PATHWAY:
            logging.info('Generating pathway network')

            network_lines, node_metadata = \
                            nb.pathway_matrix(abundances_metagenome, 
                                              abundances_transcriptome,
                                              abundances_expression,
                                              abundances_metabolome,
                                              limit,
                                              filter)

            self._write_results(os.path.join(output_directory, self.NETWORK_OUTPUT_FILE), network_lines)
            self._write_results(os.path.join(output_directory, self.METADATA_OUTPUT_FILE), node_metadata)


#!/usr/bin/env python
###############################################################################
#                                                                             #
# This program is free software: you can redistribute it and/or modify        #
# it under the terms of the GNU General Public License as published by        #
# the Free Software Foundation, either version 3 of the License, or           #
# (at your option) any later version.                                         #
#                                                                             #
# This program is distributed in the hope that it will be useful,             #
# but WITHOUT ANY WARRANTY; without even the implied warranty of              #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the                #
# GNU General Public License for more details.                                #
#                                                                             #
# You should have received a copy of the GNU General Public License           #
# along with this program. If not, see <http://www.gnu.org/licenses/>.        #
#                                                                             #
###############################################################################
 
__author__ = "Joel Boyd"
__copyright__ = "Copyright 2017"
__credits__ = ["Joel Boyd"]
__license__ = "GPL3"
__maintainer__ = "Joel Boyd"
__email__ = "joel.boyd near uq.net.au"
__status__ = "Development"
 
###############################################################################
# Imports

import os
import logging
import pickle
import inspect

###############################################################################


class Databases:
	"""Databases yo"""

	DATA_PATH   			= os.path.join(os.path.dirname(inspect.stack()[-1][1]), '..', 'share', 'enrichm')
	DATABASE_DIR			= os.path.join(DATA_PATH, 'databases')
	if os.path.isfile(os.path.join(DATABASE_DIR, 'VERSION')):
		DB_VERSION				= open(os.path.join(DATABASE_DIR, 'VERSION')).readline().strip().replace('.tar.gz','')
		CUR_DATABASE_DIR		= os.path.join(DATABASE_DIR, DB_VERSION)
		PICKLE_VERSION			= open(os.path.join(CUR_DATABASE_DIR, 'VERSION')).readline().strip()
		OLD_DATABASE_PATH		= os.path.join(DATA_PATH, 'databases', 'old')
		IDS_DIR					= os.path.join(CUR_DATABASE_DIR, 'ids')
		REF_DIR					= os.path.join(CUR_DATABASE_DIR, 'databases')
		PICKLE					= 'pickle'	
		HMM_SUFFIX 				= '.hmm'
		KO_DB_NAME				= 'uniref100'
		PFAM_DB_NAME			= 'pfam'
		TIGRFAM_DB_NAME			= 'tigrfam'

		M2DEF       			= os.path.join(CUR_DATABASE_DIR, 'module_to_definition')
		M           			= os.path.join(CUR_DATABASE_DIR, 'module_descriptions')
		COMPOUND_DESC_PICKLE	= os.path.join(CUR_DATABASE_DIR, 'br08001')    
		R2RPAIR 				= os.path.join(CUR_DATABASE_DIR, 'reaction_to_rpair')
		R2K     				= os.path.join(CUR_DATABASE_DIR, 'reaction_to_orthology')
		R2C 					= os.path.join(CUR_DATABASE_DIR, 'reaction_to_compound')
		R2M 					= os.path.join(CUR_DATABASE_DIR, 'reaction_to_module')
		M2R 					= os.path.join(CUR_DATABASE_DIR, 'module_to_reaction')
		R2P 					= os.path.join(CUR_DATABASE_DIR, 'reaction_to_pathway')
		P2R 					= os.path.join(CUR_DATABASE_DIR, 'pathway_to_reaction')
		C2R 					= os.path.join(CUR_DATABASE_DIR, 'compound_to_reaction')
		C   					= os.path.join(CUR_DATABASE_DIR, 'compound_descriptions')    
		R   					= os.path.join(CUR_DATABASE_DIR, 'reaction_descriptions')
		P   					= os.path.join(CUR_DATABASE_DIR, 'pathway_descriptions')

		PFAM2CLAN				= os.path.join(CUR_DATABASE_DIR, 'pfam_to_clan')
		CLAN2NAME				= os.path.join(CUR_DATABASE_DIR, 'clan_to_name')
		PFAM2NAME				= os.path.join(CUR_DATABASE_DIR, 'pfam_to_name')
		PFAM2DESCRIPTION		= os.path.join(CUR_DATABASE_DIR, 'pfam_to_description')
		CLAN2PFAM				= os.path.join(CUR_DATABASE_DIR, 'clan_to_pfam')

	def __init__(self):

		self.signature_modules = set(['M00611', 'M00612', 'M00613', 'M00614',
		'M00617', 'M00618', 'M00615', 'M00616', 'M00363', 'M00542', 'M00574',
		'M00575', 'M00564', 'M00660', 'M00664', 'M00625', 'M00627', 'M00745',
		'M00651', 'M00652', 'M00704', 'M00725', 'M00726', 'M00730', 'M00744',
		'M00718', 'M00639', 'M00641', 'M00642', 'M00643', 'M00769', 'M00649',
		'M00696', 'M00697', 'M00698', 'M00700', 'M00702', 'M00714', 'M00705',
		'M00746'])

		logging.info("Loading databases")
		logging.debug("Loading module definitions")
		self.m2def = pickle.load(open('.'.join([self.M2DEF,
		                                         self.PICKLE_VERSION, self.PICKLE])))
		logging.debug("Loading module descriptions")
		self.m = pickle.load(open('.'.join([self.M,
		                                    self.PICKLE_VERSION, self.PICKLE])))	
		logging.debug("Loading reaction to pathway information")
		self.r2p = pickle.load(open('.'.join([self.R2P, 
		                                      self.PICKLE_VERSION, self.PICKLE])))
		logging.debug("Loading pathway to reaction information")
		self.p2r = pickle.load(open('.'.join([self.P2R, 
		                                         self.PICKLE_VERSION, self.PICKLE])))
		logging.debug("Loading reaction to orthology information")
		self.r2k = pickle.load(open('.'.join([self.R2K, self.PICKLE_VERSION, 
		                                      self.PICKLE])))
		logging.debug("Loading reaction to module information")
		self.r2m = pickle.load(open('.'.join([self.R2M, 
		                                      self.PICKLE_VERSION, self.PICKLE])))
		logging.debug("Loading reaction to module information")
		self.m2r = pickle.load(open('.'.join([self.M2R, 
		                                      self.PICKLE_VERSION, self.PICKLE])))
		logging.debug("Loading reaction to compound information")
		self.r2c = pickle.load(open('.'.join([self.R2C, 
		                                      self.PICKLE_VERSION, self.PICKLE])))
		logging.debug("Loading compound to reaction information")
		self.c2r = pickle.load(open('.'.join([self.C2R,
		                                      self.PICKLE_VERSION, self.PICKLE])))
		logging.debug("Loading compound descriptions")
		self.c   = pickle.load(open('.'.join([self.C, 
		                                      self.PICKLE_VERSION, self.PICKLE])))
		logging.debug("Loading pathway descriptions")
		self.p   = pickle.load(open('.'.join([self.P, 
		                                      self.PICKLE_VERSION, self.PICKLE])))
		logging.debug("Loading reaction descriptions")
		self.r   = pickle.load(open('.'.join([self.R, 
		                                      self.PICKLE_VERSION, self.PICKLE])))
		logging.debug("Loading compound classifications")
		self.compound_desc_dict \
		         = pickle.load(open('.'.join([self.COMPOUND_DESC_PICKLE, 
		                                      self.PICKLE_VERSION, self.PICKLE])))
		logging.debug("Loading pfam to clan debugrmation")
		self.pfam2clan \
		         = pickle.load(open('.'.join([self.PFAM2CLAN, 
		                                      self.PICKLE_VERSION, self.PICKLE])))
		logging.debug("Loading clan descriptions")
		self.clan2name \
		         = pickle.load(open('.'.join([self.CLAN2NAME, 
		                                      self.PICKLE_VERSION, self.PICKLE])))
		logging.debug("Loading pfam descriptions")
		self.pfam2name \
		         = pickle.load(open('.'.join([self.PFAM2NAME, 
		                                      self.PICKLE_VERSION, self.PICKLE])))
		logging.debug("Loading pfam")
		self.pfam2description \
		         = pickle.load(open('.'.join([self.PFAM2DESCRIPTION, 
		                                      self.PICKLE_VERSION, self.PICKLE])))
		logging.debug("Loading pfam")
		self.clan2pfam \
		         = pickle.load(open('.'.join([self.CLAN2PFAM, 
		                                      self.PICKLE_VERSION, self.PICKLE])))
		logging.info("Loading reference db paths")		
		self.KO_DB 			= os.path.join(self.REF_DIR, self.KO_DB_NAME + self.HMM_SUFFIX)
		self.PFAM_DB 		= os.path.join(self.REF_DIR, self.PFAM_DB_NAME + self.HMM_SUFFIX)
		self.TIGRFAM_DB 	= os.path.join(self.REF_DIR, self.TIGRFAM_DB_NAME + self.HMM_SUFFIX)
		self.PFAM_CLAN_DB 	= os.path.join(self.IDS_DIR, 'PFAM_CLANS.txt')



#!/usr/bin/env python
###############################################################################
#                                                                             #
# This program is free software: you can redistribute it and/or modify        #
# it under the terms of the GNU General Public License as published by        #
# the Free Software Foundation, either version 3 of the License, or           #
# (at your option) any later version.                                         #
#                                                                             #
# This program is distributed in the hope that it will be useful,             #
# but WITHOUT ANY WARRANTY; without even the implied warranty of              #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the                #
# GNU General Public License for more details.                                #
#                                                                             #
# You should have received a copy of the GNU General Public License           #
# along with this program. If not, see <http://www.gnu.org/licenses/>.        #
#                                                                             #
###############################################################################
 
__author__ = "Joel Boyd"
__copyright__ = "Copyright 2017"
__credits__ = ["Joel Boyd"]
__license__ = "GPL3"
__maintainer__ = "Joel Boyd"
__email__ = "joel.boyd near uq.net.au"
__status__ = "Development"

###############################################################################
# Imports

import logging
import sys
import os
import shutil
import time

from data import Data
from network_analyzer import NetworkAnalyser
from enrichment import Enrichment
from annotate import Annotate
from classifier import Classify
from comparer import Compare

###############################################################################

debug={1:logging.CRITICAL,
       2:logging.ERROR,
       3:logging.WARNING,
       4:logging.INFO,
       5:logging.DEBUG}

###############################################################################

class Run:
    
    def __init__(self):
        
        self.ANNOTATE        = 'annotate'
        self.COMPARE         = 'compare'

        self.CLASSIFY        = 'classify'
        self.BUILD           = 'build'
        self.ENRICHMENT      = 'enrichment'
        self.MODULE_AB       = 'module_ab'
        self.DATA            = 'data'

    def _logging_setup(self, args):
        logger = logging.getLogger('')
        logger.setLevel(debug[args.verbosity])
        log_format = logging.Formatter(fmt="[%(asctime)s] %(levelname)s: %(message)s",
                                       datefmt="%Y-%m-%d %H:%M:%S %p")

        stream_logger = logging.StreamHandler(sys.stdout)
        stream_logger.setFormatter(log_format)
        stream_logger.setLevel(debug[args.verbosity])
        logger.addHandler(stream_logger)
        if args.subparser_name!=self.DATA:


            file_logger = logging.FileHandler(os.path.join(args.output, args.log), 'a')
            file_logger.setFormatter(log_format)
            logger.addHandler(file_logger)

    def _check_general(self, args):
        '''
        Check general input and output options are valid.

        Parameters
        ----------
        args    - object. Argparse object
        '''

        if args.subparser_name!=self.DATA:
            # Set up working directory
            if not args.output:
                args.output = '%s-enrichm_%s_output' % (time.strftime("%Y-%m-%d_%H-%M"), args.subparser_name)

            if(os.path.isdir(args.output) or os.path.isfile(args.output)):
                if args.force:
                    if os.path.isdir(args.output):
                        shutil.rmtree(args.output)
                    else:
                        os.remove(args.output)
                else:
                    raise Exception("File '%s' exists." % args.output)

            os.mkdir(args.output)   

    def _check_data(self, args):
        '''
        Check data input and output options are valid.

        Parameters
        ----------
        args    - object. Argparse object
        '''
        pass

    def _check_annotate(self, args):
        '''
        Check annotate input and output options are valid.

        Parameters
        ----------
        args    - object. Argparse object
        '''
        # ensure either a list of genomes or a directory of genomes have been specified
        if not(args.genome_files or args.genome_directory or args.protein_directory or args.protein_files):
            raise Exception("Input error: Either a list of genomes or a directory of genomes need to be specified.")
        if len([x for x in [args.genome_files, args.genome_directory, args.protein_directory, args.protein_files] if x]) != 1:
            raise Exception("Input error: Only one type of input can be specified (--genome_files, --genome_directory, --protein_directory, or --protein_files).")
        if(args.genome_directory or args.genome_files):
            args.suffix = '.fna'
        elif(args.protein_directory or args.protein_files):
            args.suffix = '.faa'

    def _check_enrichment(self, args):
        '''
        
        Parameters
        ----------
        
        Output
        ------
        '''
        if not any([args.do_all, args.do_gvg, args.do_ivg, args.do_ivi]):
            raise Exception("Input error: No comparisons were specified. You will need to tell enrichM \
which statistical tests to run using the --do_ivi --do_gvg --do_ivg, or --do_all flags")
        ### ~ TODO: Check Multi test correction inputs...
        if not(args.annotation_matrix or args.annotation_file):
            raise Exception("Input error: No input file was specified. Please specify annotations to either the --annotation_matrix --annotation_file flags")
    
    def _check_classify(self, args):  
        '''
        Check classify input and output options are valid.
        
        Parameters
        ----------
        args    - object. Argparse object
        '''
        # Ensure either an annotation matrix or list file has been specified:
        if not(args.genome_and_annotation_file or args.genome_and_annotation_matrix):
            raise Exception("Input error: An input file must be specified to either \
--genome_and_annotation_file or --genome_and_annotation_matrix")

    def _check_build(self, args):
        '''
        Check build input and output options are valid.
        
        Parameters
        ----------
        args    - object. Argparse object
        '''
        if not(args.metadata or args.abundances):
            raise Exception("No metadata or abundance information provided to build.")
    
    def _check_compare(self, args):
        pass ### ~ TODO: Dunno yet
    
    def _check_network(self, args):
        '''
        Check network (explore, pathway) input and output options are valid.
        
        Parameters
        ----------
        args    - object. Argparse object
        '''

        if args.subparser_name==NetworkAnalyser.EXPLORE:
            args.filter             = None  
            args.limit              = None 
            args.starting_compounds = None
            args.steps              = None
            args.number_of_queries  = None

            if not(args.queries):
                if args.depth:
                    logging.warning("--depth argument ignored without --queries flag")

        if args.subparser_name==NetworkAnalyser.PATHWAY:
            args.depth              = None
            args.queries            = None
            args.starting_compounds = None
            args.steps              = None
            args.number_of_queries  = None
        
        if args.subparser_name==NetworkAnalyser.TRAVERSE:
            args.depth              = None
            args.queries            = None


    def main(self, args, command):
        '''
        Parameters
        ----------
        
        Output
        ------
        '''

        self._check_general(args)
        self._logging_setup(args)

        logging.info("Running command: %s" % ' '.join(command))

        if args.subparser_name == self.DATA:
            self._check_data(args)
            d = Data()

            d.do()
        
        if args.subparser_name == self.ANNOTATE:
            self._check_annotate(args)
            a = Annotate(# Define inputs and outputs
                         args.output,
                         # Define type of annotation to be carried out
                         args.ko,
                         args.pfam,
                         args.tigrfam,
                         args.cog,
                         # Cutoffs
                         args.evalue,
                         args.bit,
                         args.id,
                         args.aln_query, 
                         args.aln_reference, 
                         # Parameters
                         args.threads,
                         args.suffix)

            a.do(args.genome_directory, args.protein_directory, 
                 args.genome_files, args.protein_files)

        elif args.subparser_name == self.CLASSIFY:
            self._check_classify(args)
            c = Classify()
            c.do(args.custom_modules, 
                 args.cutoff,
                 args.genome_and_annotation_file,
                 args.genome_and_annotation_matrix,
                 args.output)

        elif args.subparser_name == self.ENRICHMENT: 
            self._check_enrichment(args)
            e = Enrichment()
            e.do(# Inputs
                 args.annotation_matrix,
                 args.annotation_file,
                 args.metadata,
                 args.modules,
                 args.abundances,
                 args.do_all,
                 args.do_ivi, 
                 args.do_gvg,
                 args.do_ivg,
                 args.pval_cutoff,
                 args.proportions_cutoff,
                 args.threshold,
                 args.multi_test_correction,
                 args.output)

        elif args.subparser_name == self.COMPARE:
            self._check_compare(args)
            c = Compare()
            c.do(args.enrichm_annotate_output)

        elif(args.subparser_name == NetworkAnalyser.PATHWAY or
             args.subparser_name == NetworkAnalyser.EXPLORE or
             args.subparser_name == NetworkAnalyser.TRAVERSE):

            self._check_network(args)
            na=NetworkAnalyser(args.metadata)
            na.do(args.matrix,
                  args.transcriptome,
                  args.metabolome,
                  args.depth,
                  args.filter,
                  args.limit,
                  args.queries,
                  args.subparser_name,
                  args.starting_compounds, 
                  args.steps,
                  args.number_of_queries,
                  args.output)
        logging.info('Done!')
